/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.commons.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

/**
 * Utility class containing helpful methods related to manipulating Collections.
 */
public final class CollectionUtils {

    /**
     * Converts the values to a {@link List}.
     * 
     * @param values
     *            the values to convert to a {@link List}, can be <code>null</code>.
     * @return a {@link List} of the supplied values, which may be empty, but never <code>null</code>.
     */
    public static <T> List<T> asList(T... values) {
        List<T> result = new ArrayList<T>();
        if (values != null) {
            for (T tmp : values) {
                result.add(tmp);
            }
        }
        return result;
    }

    /**
     * Converts the values to a {@link Set}.
     * 
     * @param values
     *            the values to convert to a {@link Set}, can be <code>null</code>.
     * @return a {@link Set} of the supplied values, which may be empty, but never <code>null</code>.
     */
    public static <T> Set<T> asSet(T... values) {
        Set<T> result = new HashSet<T>();
        if (values != null) {
            for (T tmp : values) {
                result.add(tmp);
            }
        }
        return result;
    }

    public static <T> List<T> filter(Collection<T> collection, Predicate<T> predicate) {
        List<T> result = new ArrayList<T>();
        for (T element : collection) {
            if (predicate.evaluate(element)) {
                result.add(element);
            }
        }
        return result;
    }

    public static <K, V> Map<K,V> filter(Map<K,V> source, Predicate<Map.Entry<K,V>> predicate) {
        Map<K,V> result = new HashMap<K,V>();
        for (Map.Entry<K,V> entry : source.entrySet()) {
            if (predicate.evaluate(entry)) {
                result.put(entry.getKey(), entry.getValue());
            }
        }
        return result;
    }

    public static <T> T getFirstMatchingElement(Collection<T> collection, Predicate<? super T> predicate) {
        T result = null;
        for (T element : collection) {
            if (predicate.evaluate(element)) {
                result = element;
                break;
            }
        }
        return result;
    }

    public static <K, V> K getKeyByValue(Map<K,V> map, V value) {
        K result = null;
        for (Map.Entry<K,V> entry : map.entrySet()) {
            if (value.equals(entry.getValue())) {
                result = entry.getKey();
            }
        }
        return result;
    }

    public static <T> T getLastMatchingElement(Collection<T> collection, Predicate<? super T> predicate) {
        T result = null;
        for (T element : collection) {
            if (predicate.evaluate(element)) {
                result = element;
            }
        }
        return result;
    }

    public static <T extends Object> List<T> removeDuplicates(List<T> list) {
        return removeDuplicates(list, null);
    }

    public static <T extends Object> List<T> removeDuplicates(List<T> list, Comparator<T> comparator) {
        if (list == null) {
            throw new IllegalArgumentException("parameter list cannot be null");
        }
        Set<T> extensionSet = new TreeSet<T>(comparator);
        extensionSet.addAll(list);
        return toList(extensionSet);
    }

    /**
     * Creates a new {@link java.util.List} object based on a {@link java.util.Set} object.
     * 
     * @param set
     *            the source Collection.
     * @return a new Collection containing all of the elements from the source collection.
     * @param <T>
     *            a T object.
     */
    public static <T extends Object> List<T> toList(Set<T> set) {
        List<T> list = new ArrayList<T>();
        list.addAll(set);
        return list;
    }

    /**
     * Creates a new {@link java.util.Set} object based on a {@link java.util.List} object.
     * 
     * @param <T>
     * @param list
     *            the source Collection.
     * @return a new Collection containing only the unique elements from the source Collection.
     */
    public static <T extends Object> Set<T> toSet(List<T> list) {
        Set<T> set = new HashSet<T>();
        set.addAll(list);
        return set;
    }

    public static <K, V> List<V> values(Map<K,V> source) {
        List<V> result = new ArrayList<V>();
        for (Map.Entry<K,V> entry : source.entrySet()) {
            result.add(entry.getValue());
        }
        return result;
    }

    /**
     * Hide the constructor because this is a utility class.
     */
    private CollectionUtils() {
    }

}
