/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.commons.util;

import java.awt.Color;
import java.awt.color.ColorSpace;
import java.awt.color.ICC_ColorSpace;
import java.awt.color.ICC_Profile;
import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Utility class to get Color information such as hex, rbg and cmyk values and to do conversions.
 */
public final class ColorUtils {

    private static final Logger LOG = LoggerFactory.getLogger(ColorUtils.class);
    private static ICC_Profile cmykProfile;
    private static ColorSpace cmykSpace;
    private static ColorUtils instance;
    private static double[] refXyz;
    private static ICC_Profile rgbProfile;
    private static ColorSpace rgbSpace;

    /**
     * No args constructor for creating singleton instance.
     */
    private ColorUtils() {
    }

    /**
     * Int[4] each 0 - 100 (%).
     * 
     * @param lab
     *            lab value
     * @return int[4] each 0 - 100 (%)
     */
    public static int[] getCmyk(double[] lab) {
        return getCmyk(new float[] {(float) lab[0], (float) lab[1], (float) lab[2]});
    }

    /**
     * Java.awt.Color.
     * 
     * @param lab
     *            lab value
     * @return java.awt.Color
     */
    public static Color getColor(double[] lab) {
        return getColor(new float[] {(float) lab[0], (float) lab[1], (float) lab[2]});
    }

    /**
     * Int[3] each 0 - 255 (byte).
     * 
     * @param lab
     *            lab value
     * @return int[3] each 0 - 255 (byte)
     */
    public static int[] getRgb(double[] lab) {
        return getRgb(new float[] {(float) lab[0], (float) lab[1], (float) lab[2]});
    }

    /**
     * Six digit hex string (upper case).
     * 
     * @param lab
     *            lab value
     * @return six digit hex string (upper case)
     */
    public static String getRgbHex(double[] lab) {
        return getRgbHex(new float[] {(float) lab[0], (float) lab[1], (float) lab[2]});
    }

    /**
     * Int[4] each 0 - 100 (%).
     * 
     * @param lab
     *            lab value
     * 
     * @return int[4] each 0 - 100 (%)
     */
    private static int[] getCmyk(float[] lab) {
        float[] xyz = ColorUtils.labToXyz(lab);
        float[] cmyk = cmykSpace.fromCIEXYZ(xyz);

        return new int[] {Math.round(cmyk[0] * 100), Math.round(cmyk[1] * 100), Math.round(cmyk[2] * 100), Math.round(cmyk[3] * 100)};
    }

    /**
     * Java.awt.Color.
     * 
     * @param lab
     *            lab value
     * 
     * @return java.awt.Color
     */
    private static Color getColor(float[] lab) {
        float[] xyz = ColorUtils.labToXyz(lab);
        float[] rgb = rgbSpace.fromCIEXYZ(xyz);

        return new Color(rgb[0], rgb[1], rgb[2], 1.0f);
    }

    /**
     * Int[3] each 0 - 255 (byte).
     * 
     * @param lab
     *            lab value
     * 
     * @return int[3] each 0 - 255 (byte)
     */
    private static int[] getRgb(float[] lab) {
        float[] xyz = ColorUtils.labToXyz(lab);
        float[] rgb = rgbSpace.fromCIEXYZ(xyz);

        return new int[] {Math.round(rgb[0] * 255), Math.round(rgb[1] * 255), Math.round(rgb[2] * 255)};
    }

    /**
     * Six digit hex string (upper case).
     * 
     * @param lab
     *            lab value
     * 
     * @return six digit hex string (upper case)
     */
    private static String getRgbHex(float[] lab) {
        Color color = getColor(lab);
        String rgbHex = Integer.toHexString(color.getRGB() << 8 >>> 8).toUpperCase();
        while (rgbHex.length() < 6) {
            rgbHex = "0" + rgbHex;
        }

        return rgbHex;
    }

    /**
     * Private worker method to do the math!
     * 
     * @param rawLab
     *            LAB values (ranges are about 1-100, not 0-1)
     * 
     * @return float[] CIE XYZ values (from 0-1)
     */
    private static float[] labToXyz(float[] rawLab) {
        float[] result = null;
        if (rawLab != null) {
            if (instance == null) {
                instance = new ColorUtils();
                instance.init();
            }

            double[] lab = new double[] {(double) rawLab[0], (double) rawLab[1], (double) rawLab[2]};
            double[] xyz = new double[3];

            xyz[1] = (lab[0] + 16) / 116;
            xyz[0] = (lab[1] / 500) + xyz[1];
            xyz[2] = xyz[1] - (lab[2] / 200);

            if (Math.pow(xyz[1], 3) > 0.008856) {
                xyz[1] = (float) Math.pow(xyz[1], 3);
            } else {
                xyz[1] = (float) (xyz[1] - (16 / 116)) / 7.787;
            }

            if (Math.pow(xyz[0], 3) > 0.008856) {
                xyz[0] = (float) Math.pow(xyz[0], 3);
            } else {
                xyz[0] = (float) (xyz[0] - (16 / 116)) / 7.787;
            }

            if (Math.pow(xyz[2], 3) > 0.008856) {
                xyz[2] = (float) Math.pow(xyz[2], 3);
            } else {
                xyz[2] = (float) (xyz[2] - (16 / 116)) / 7.787;
            }

            xyz[0] = (float) (refXyz[0] * xyz[0]);
            xyz[1] = (float) (refXyz[1] * xyz[1]);
            xyz[2] = (float) (refXyz[2] * xyz[2]);

            result = new float[] {(float) xyz[0], (float) xyz[1], (float) xyz[2]};
        }
        return result;

    }

    /**
     * Private worker method to initialize state only when needed.
     */
    private void init() {
        try {
            rgbProfile = ICC_Profile.getInstance(getClass().getResourceAsStream("/resources/rgb-color.icm"));
            cmykProfile = ICC_Profile.getInstance(getClass().getResourceAsStream("/resources/cmyk-color.icm"));

            // Illuminant from "rgb-color.icm" file we are using above
            // Note: these values must match the profile used for accurate results!
            // (use a profile inspector to determine when changing profiles)
            refXyz = new double[] {0.96419, 1.00000, 0.82489};
            rgbSpace = new ICC_ColorSpace(rgbProfile);

            cmykSpace = new ICC_ColorSpace(cmykProfile);
        } catch (IOException e) {
            LOG.error(e.getMessage(), e);
        }
    }
}
