/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.commons.util;

import java.util.Collection;
import java.util.Iterator;

public class FilteredCollection<E> implements Collection<E> {

    private Predicate<E> filter;
    private Collection<E> source;

    public FilteredCollection(Collection<E> source, Predicate<E> filter) {
        this.source = source;
        this.filter = filter;
    }

    @Override
    public boolean add(E element) {
        boolean result = false;
        if (filter.evaluate(element)) {
            result = source.add(element);
        }
        return result;
    }

    @Override
    public boolean addAll(Collection<? extends E> elements) {
        boolean result = false;
        for (E element : elements) {
            if (filter.evaluate(element) && source.add(element)) {
                result = true;
            }
        }
        return result;
    }

    @Override
    public void clear() {
        source.clear();
    }

    @Override
    public boolean contains(Object obj) {
        // per the JavaDocs on the Collection class, it's OK to throw a ClassCastException here
        @SuppressWarnings("unchecked") E element = (E) obj;
        return filter.evaluate(element) && source.contains(element);
    }

    @Override
    public boolean containsAll(Collection<?> objs) {
        boolean result = true;
        // per the JavaDocs on the Collection class, it's OK to throw a ClassCastException here
        @SuppressWarnings("unchecked") Collection<E> elements = (Collection<E>) objs;
        for (E element : elements) {
            if (!filter.evaluate(element) ||!source.contains(element)) {
                result = false;
                break;
            }
        }
        return result;
    }

    @Override
    public Iterator<E> iterator() {
        return new FilteredIterator<E>(source.iterator(), filter);
    }

    @Override
    public boolean remove(Object obj) {
        // per the JavaDocs on the Collection class, it's OK to throw a ClassCastException here
        @SuppressWarnings("unchecked") E element = (E) obj;
        return filter.evaluate(element) && source.remove(element);
    }

    @Override
    public boolean removeAll(Collection<?> objs) {
        boolean result = false;
        // per the JavaDocs on the Collection class, it's OK to throw a ClassCastException here
        @SuppressWarnings("unchecked") Collection<E> elements = (Collection<E>) objs;
        for (E element : elements) {
            if (filter.evaluate(element) && source.remove(element)) {
                result = true;
            }
        }
        return result;
    }

    @Override
    public boolean retainAll(Collection<?> objs) {
        boolean result = false;
        // per the JavaDocs on the Collection class, it's OK to throw a ClassCastException here
        @SuppressWarnings("unchecked") Collection<E> retainElements = (Collection<E>) objs;
        for (E element : source) {
            if (filter.evaluate(element) &&!retainElements.contains(element)) {
                source.remove(element);
            }
        }
        return result;
    }

    @Override
    public int size() {
        int validElementCount = 0;
        for (E element : source) {
            if (filter.evaluate(element)) {
                validElementCount++;
            }
        }
        return validElementCount;
    }

    @Override
    public Object[] toArray() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public <T> T[] toArray(T[] arg0) {
        // TODO Auto-generated method stub
        return null;
    }

    protected Predicate<E> getFilter() {
        return filter;
    }

    @Override
    public boolean isEmpty() {
        return size() == 0;
    }
}
