/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.commons.xml;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;

/**
 * <p>LoggingContentHandler class.</p>
 */
public class LoggingContentHandler implements ContentHandler {

    private Logger logger;

    /**
     * <p>Constructor for LoggingContentHandler.</p>
     */
    public LoggingContentHandler() {
        logger = LoggerFactory.getLogger(LoggingContentHandler.class.getName());
    }

    public LoggingContentHandler(Logger logger) {
        this.logger = logger;
    }

    /**
     * <p>Getter for the field <code>logger</code>.</p>
     *
     * @return a {@link java.util.logging.Logger} object.
     */
    public Logger getLogger() {
        return logger;
    }

    /**
     * <p>Setter for the field <code>logger</code>.</p>
     *
     * @param logger a {@link java.util.logging.Logger} object.
     */
    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    /** {@inheritDoc} */
    @Override
    public void setDocumentLocator(Locator locator) {
        logger.info("setDocumentLocator - locator : {}", locator);
    }

    /** {@inheritDoc} */
    @Override
    public void startDocument() {
        logger.info("startDocument");
    }

    /** {@inheritDoc} */
    @Override
    public void endDocument() {
        logger.info("endDocument");
    }

    /** {@inheritDoc} */
    @Override
    public void startPrefixMapping(String prefix, String uri) {
        logger.info("startPrefixMapping - prefix : {}, url : {}", new Object[] {prefix, uri});
    }

    /** {@inheritDoc} */
    @Override
    public void endPrefixMapping(String prefix) {
        logger.info("endPrefixMapping - prefix : {}", prefix);
    }

    /** {@inheritDoc} */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) {
        logger.info("startElement - uri : {}, localName : {}, qName : {}, atts : {}", new Object[] {uri, localName, qName, atts});
    }

    /** {@inheritDoc} */
    @Override
    public void endElement(String uri, String localName, String qName) {
        logger.info("endElement - uri : {}, localName : {}, qName : {}", new Object[] {uri, localName, qName});
    }

    /** {@inheritDoc} */
    @Override
    public void characters(char[] chars, int start, int length) {
        logger.info("characters - chars : {}, start : {}, length : {}", new Object[] {String.valueOf(chars, start, length), start, length});
    }

    /** {@inheritDoc} */
    @Override
    public void ignorableWhitespace(char[] chars, int start, int length) {
        logger.info("ignorableWhitespace - chars : {}, start : {}, length : {}", new Object[] {String.valueOf(chars, start, length), start, length});
    }

    /** {@inheritDoc} */
    @Override
    public void processingInstruction(String target, String data) {
        logger.info("processingInstruction - target : {}, data : {}", new Object[] {target, data});
    }

    /** {@inheritDoc} */
    @Override
    public void skippedEntity(String name) {
        logger.info("skippedEntity - name : {}", name);
    }
}
