/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.commons.xml;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Provides common XML related functionality.
 */
public final class XmlUtils {

    private static final Logger LOG = LoggerFactory.getLogger(XmlUtils.class);

    private XmlUtils() {
    }

    /**
     * Method to convert illegal XML characters (&amp;, &lt;, &gt;, &apos;, &quot;) into corresponding XML entity
     * strings (&amp;amp;, &amp;lt;, &amp;gt;, &amp;apos;, &amp;quot;).
     *
     * @param   plainString  instance to encode
     * @return  encoded result or <code>null</code> if the passed in parameter was <code>null</code>.
     */
    public static String escapeXmlEntities(String plainString) {
        String result = null;
        if (plainString != null) {
            StringBuilder sb = new StringBuilder();
            for (char c : plainString.toCharArray()) {
                switch (c) {
                    case '&' :
                        sb.append("&amp;");
                        break;
                    case '<' :
                        sb.append("&lt;");
                        break;
                    case '>' :
                        sb.append("&gt;");
                        break;
                    case '\'' :
                        sb.append("&apos;");
                        break;
                    case '"' :
                        sb.append("&quot;");
                        break;
                    default :
                        sb.append(c);
                }
            }
            result = sb.toString();
        }
        LOG.debug("original: \"{}\", escaped: \"{}\"", plainString, result);
        return result;
    }

    /**
     * <p>unescapeXmlEntities.</p>
     *
     * @param xmlString a {@link java.lang.String} object.
     * @return a {@link java.lang.String} object.
     */
    public static String unescapeXmlEntities(String xmlString) {
        String result = xmlString;
        if (result != null) {
            result = result.replace("&quot;", "\"");
            result = result.replace("&lt;", "<");
            result = result.replace("&gt;", ">");
            result = result.replace("&amp;", "&");
            result = result.replace("&nbsp;", "" + (char) 160);
        }
        LOG.debug("original: \"{}\", unescaped: \"{}\"", xmlString, result);
        return result;
    }
}
